require 'rack/request'
require 'rack/response'

require 'rau/response_handler'

module Rau
  class Route
    attr_reader :method, :path_info

    def initialize method, path_info
      @method = method
      @path_info = path_info
    end

    def == other
      (@method == other.method) && \
        (@path_info == other.path_info)
    end

    alias eql? ==

    def hash
      @method.hash ^ @path_info.hash
    end
  end

  module App
    def self.included(base)
      base.extend(DSL)
    end

    def call env
      @request = Rack::Request.new env
      @response = Rack::Response.new

      route = Route.new @request.request_method, @request.path_info

      block = routes[route]

      if block.nil?
        @response.status = 404
      else
        @out = []
        instance_eval &block
        @response.write @out.join("\n")
      end

      @response.headers['Content-Type'] = 'text/html'
      @response.finish
    end

    def display_form name
      HTML::FormPrinter.new @out do |p|
        p.print forms[name]
      end
    end
  end

  module DSL
    def self.extended(app_class)
      app_class.instance_eval do
        @@forms = Hash.new
        @@routes = Hash.new

        define_method :routes do
          @@routes
        end

        define_method :forms do
          @@forms
        end
      end
    end

    def form name, action, &block
      @@forms[name] = Rau::Form.new name, action, &block
    end

    def receive form_name, destination, &block
      route = Route.new 'POST', @@forms[form_name].action
      @@routes[route] = Proc.new do
        block.call @request.POST
        @response.redirect destination
      end
    end

    def get path, &block
      route = Route.new 'GET', path
      @@routes[route] = block
    end
  end
end
