module Rau
  class Field
    attr_reader :type, :name, :label

    def initialize type, name, label
      @type = type
      @name = name
      @label = label
    end
  end

  class Form
    attr_reader :name, :action, :fields

    def initialize name, action, &block
      @name = name
      @action = action

      @fields = []

      if block
        instance_eval &block
      end
    end

    def input params
      @params = params
    end

    # The DSL should have no behavior.
    #
    # It is only a convenient wrapper around what would otherwise amount
    # to manually instantiating objects such as Rau::Field.
    #
    # This way, things can be automated more easily later on.

    FIELD_TYPES = [
      :text,
      :email,
      :password,
      :submit
    ]

    FIELD_TYPES.each do |type|
      define_method type do |name, label|
        @fields.push Field.new(type, name, label)
      end
    end
  end
end
