module Rau
  module HTML
    class Printer
      attr_reader :out
      def initialize out = nil
        if out.nil?
          @out = []
        else
          @out = out
        end

        yield self
      end
    end

    class FormPrinter < Printer
      def print form
        @out.push %Q[<form id="#{form.name}" action="#{form.action}" method="POST">]

        FieldPrinter.new @out do |p|
          form.fields.each do |field|
            p.print form, field
          end
        end

        @out.push %Q[</form>]
      end
    end

    class FieldPrinter < Printer
      FIELD_TYPE_TAG_MAP = {
        :submit => [:button_field, 'submit'],
        :email => [:wrapped_input, 'text'],
        :password => [:wrapped_input, 'password'],
        :text => [:wrapped_input, 'text']
      }

      def print form, field
        @form = form
        @field = field
        method_name, args = FIELD_TYPE_TAG_MAP[field.type]
        send(method_name, *args)
      end

      def button_field type
        @out.push %Q[<button type="#{type}" name="#{@field.name}" id="#{@field.name}">#{@field.label}</button>]
      end

      def wrapped_input type
        @out.push %Q[<div class="input-wrapper" id="#{@form.name}-#{@field.name}-wrapper">]
        @out.push %Q[<label for="#{@field.name}">#{@field.label}</label>]
        @out.push %Q[<input type="#{type}" id="#{@field.name}" name="#{@field.name}" />]
        @out.push %Q[</div>]
      end
    end
  end
end
